#!/usr/bin/perl

###############################################################################
# build.cgi                                                                   #
###############################################################################
#
# ImageFolio v 2.1 by BizDesign, Inc.
# written by Greg Raaum, webmaster@imageFolio.com
# Available from http://www.ImageFolio.com
# ---------------------------------------------------------------------------
# PROGRAM NAME : ImageFolio 2.1
# VERSION : 2.1
# LAST MODIFIED : 04/17/2000
# ===========================================================================
# COPYRIGHT NOTICE :
#
# Copyright (c) 1999-2000 BizDesign, Inc. All Rights Reserved.
# Selling the code for this program without prior written consent is
# expressly forbidden.
#
# Obtain written permission before redistributing this software over the 
# Internet or in any other medium.  In all cases copyright and header must
# remain intact.
#
# Feel free to modify the code of this program to suit your likings. 
#
# Although this program has been thoroughly tested on BizDesign's servers, we
# do not warrant that it works on all servers and will not be held liable
# for anything, including but not limited to, misusage, error, or loss of data.
#
# Use at your own risk!
###############################################################################
# build.cgi - when called via Telnet or a Web browser, build.cgi will spider and
# index your entire imageFolio image collection for use in searches. Telnet works
# best as Web browser's typically will timeout when build.cgi tries to index a
# 10,000+ image collection.  If you update your gallery frequently, consider
# making A Cron entry to have this script to run automatically at scheduled
# intervals.  Ask your system administrator for more info on Cron.
#
# Do not modify below this line unless you know what you are doing.
###############################################################################

eval {
    ($0 =~ m,(.*)/[^/]+,)   && unshift (@INC, "$1");    # Get the script location: UNIX
    ($0 =~ m,(.*)\\[^\\]+,) && unshift (@INC, "$1");    # Get the script location: Windows

    require "../config.pl";               # Change this to full path to config.pl if you have problems.
    &build;
};

if ($@) { print "Content-type: text/plain\n\nError including configuration file: $@\n"; }
exit;


# No editing is necessary
###############################################################################

sub build {
print "Content-type: text/plain\n\n";

print "Now building the Image Index...\n\n";

    $slash = '/';

   unless (-e $image_directory) {
      print "Fatal Error!\n";
      print "Searched for a directory at specified location:\n";
      print "    $image_directory\n";
      print "No directory found. Check settings.\n";
      exit;
   }

print "First, lets open up the current Image Index and store all the lines containing keywords\n";

if (-e $indexfile) {
open (FILE,"$indexfile") || die "Content-type: text/html\n\nCan't Open $indexfile, is it created?: $!\n";
my(@stuff)=<FILE>;
$SIZE = @stuff;
close(FILE);

   for ($i=0;$i<=$SIZE;$i++) {
      my ($a, $b, $c, $d, $keywords) = split(/\|/, $stuff[$i]);
      $keywords =~ s/\n//g;
      if ($keywords) {
         push(@keylines,"$a\|$b\|$c\|$d\|$keywords");
         print "- $a ($b) has keywords\n";
         $found_keywords = 1;
         $number_found++;
      }
   }

   if (!$found_keywords) { print "None found..."; }
   else { print "$number_found found..."; }
   print "Done!\n\n";
}
else {
   print "Index file was not found, will attempt to create one.\n";
   open (INDEX,">$indexfile") || die ("Unable to create index.txt file.  Check Permissions on your Data Directory:", "$!");
   print INDEX "";
   close(INDEX);
   chmod(0777,"$indexfile");
   print "Index file created succesfully!\n\n";
}

print "Walking through directory tree...\n";

@directories = ($image_directory);
$totaldir = 1;

for ($i=0; $i<$totaldir; $i++) {
   $directory = @directories[$i];
   opendir(DIR,$directory);
   @listings = readdir(DIR);
   closedir(DIR);

   foreach $listing (@listings) {
      next if (($listing eq ".") || ($listing eq "..") || ($listing =~ /_vti_cnf/i));
      $fullpath = $directory . $slash . $listing;
      if (-d $fullpath) {
         push(@directories,$fullpath);
         $totaldir++;
      }
      elsif (-B $fullpath) {
         if ($fullpath =~ /$mypattern/i) { push(@THUMBS,$fullpath); }
      }
   }
}

$Number_Files = @THUMBS;
$Number_Cats = @directories;
$delimiter = "|";

open(ALLFILES,">$indexfile") || &Fatal;
flock(ALLFILES, 2);

   foreach $THUMB (@THUMBS) {
      $thumbnail = $THUMB;
      $thumbnail =~ s/\\/\//g;
      if ($thumbnail =~ /^$image_directory(.*)/) { $thumbnail = $1; }	
      $name = &get_filename ($thumbnail);
      $thispath = &get_path ($thumbnail);	
      print ALLFILES $name . $delimiter . $thispath . $delimiter . $delimiter . $delimiter . "\n";
      print "Captured : $name - $thispath\n";
   }


flock(ALLFILES, 8);
close(ALLFILES);


print "\nSuccessfully completed building of the Image Index.\n";
print "Captured $Number_Files images total.\n\n";

if ($found_keywords) {
print "Now Restoring Keywords in the Image Index.\n";

   foreach $keyline (@keylines) {
      my($image, $category, $blah, $blah, $words) = split(/\|/, $keyline);

      open (FILE,"$indexfile") || die "Content-type: text/html\n\nCan't Open $indexfile, is it created?: $!\n";
      my(@LINES)=<FILE>;
      close(FILE);
      $SIZE=@LINES;

      open (INDEX,">$indexfile") || die "Content-type: text/html\n\nCan't Open $indexfile, is it created?: $!\n";
      flock(INDEX, 2);

      for ($i=0;$i<=$SIZE;$i++) {
         $_=$LINES[$i];
         my ($filename, $path, $blah, $blah, $blah) = split(/\|/, $_);

         if ($image eq $filename && $category eq $path) {
            print "Restoring keywords for $filename ($path)";
            print INDEX "$filename\|$path\|\|\|$words\n";
            print "...Done!\n";
         }
         else {
            print INDEX $_;
         }
      }

      flock(INDEX, 8);
      close(INDEX);
   }
}

$Index_Size = -s $indexfile;
print "Image Index file is $Index_Size bytes.\n\n";
print "Now building the Category Index...\n\n";

@directories = sort @directories;

   foreach $catdirect (@directories) {
      $totalimages = 0;
      next if $catdirect eq $image_directory;

      chdir(IMAGEDIR,"$catdirect");
      opendir(IMAGEDIR,"$catdirect")|| die("Error Reading Directory:", "$!");
      @files = readdir(IMAGEDIR);
      foreach $file (@files) {
         if ( -d "$catdirect/$file" ) {            # if this is a directory we aren't doing anything
            # do nothing
         }
         else{
            if ( -B $file && $file =~ /$mypattern/i ) {            # if this is a thumbnail and binary file
               $totalimages++;                                     # count all of the thumnails found
               $Grand_Total++;
            }
         }
      }
      closedir(IMAGEDIR);

      $newcatdirect = $catdirect;
      $newcatdirect =~ s/$image_directory\///g;
      $catlist .= $newcatdirect . "\t" . $totalimages . "\n";
      print "Captured : $newcatdirect : $totalimages images\n";
   }

open(CATEGORIES,">$catfile") || &Fatal;
flock(CATEGORIES, 2);
print CATEGORIES $catlist;
flock(CATEGORIES, 8);
close(CATEGORIES);


print "\nSuccessfully completed building of the Category Index.\n";
print "Captured $Number_Cats categories total and $Grand_Total images total.\n";
$Cat_Size = -s $catfile;
print "Category Index file is $Cat_Size bytes.\n\n";
}

###############################################################################
# GET FILENAME
# Returns a string of the current path containing the file name broken up 
# Returns a string containing the actual filename
###############################################################################
sub get_filename {
my $input = shift;
my (@categories, $cat, $output, $path, $last);
@categories = split (/$slash/, $input);
$last = pop @categories;
$output = $last;
return $output;
}

###############################################################################
# GET PATH
# Returns a string containing path information prior to the current directory
###############################################################################
sub get_path {
my $input = shift;
#$input =~ s/\/$name//g;
my (@categories, $cat, $output, $path, $last);
@categories = split (/$slash/, $input);
$i=0;
$blah = pop @categories;
$last = pop @categories;

foreach $cat (@categories) {
  if ($i <= 1) { $path .= $cat; }
  else { $path .= $slash . $cat; }
$i++;
}
if ($path) {
$output = $path . $slash . $last;
}
else {
$output = $last;
}    
return $output;
}

###############################################################################
# FATAL
# Returns an error message if we were unable to build the index.
###############################################################################
sub Fatal {
print "Fatal Error! Cannot write to index file.\n";
print "This script does not have permission to modify the file.\n";
print "Build Module aborted.\n\n";
print "Hint: Try: chmod 777n\n";
exit;
}